; -----------------------------------------------------------------------------
; IZX driver - Copyright (c) 2013 Einar Saukas
; IZX format - Copyright (c) 2013 Andrew Owen & Einar Saukas
; -----------------------------------------------------------------------------

        org     55000                   ; driver address

ICONS   EQU     55200                   ; icons address


; -----------------------------------------------------------------------------
; ENTRY POINT #1 - TO BE ACCESSED FROM BASIC THROUGH USER-DEFINED FUNCTION
;
; 100 DEF FN z(r,c,i,p)=USR 55000
; ...
; 150 RANDOMIZE FN z(row,column,icon,clip)
;
; Notes:
;   FN z(r,c,i,p) always returns 0
; -----------------------------------------------------------------------------

; load user defined function arguments
        ld      ix, (23563)             ; system variable DEFADD
        ld      b, (ix+4)               ; 1st argument (row)
        ld      c, (ix+12)              ; 2nd argument (col)
        ld      e, (ix+20)              ; 3rd argument (icon)
        exx
        ld      b, (ix+28)              ; 4th argument (clip)
        exx


; -----------------------------------------------------------------------------
; ENTRY POINT #2 - CAN BE DIRECTLY ACCESSED FROM ASSEMBLY ROUTINES
;
; Params:
;   B = row (0-20)
;   C = column (0-28)
;   E = icon (0-255)
;   B' = clip (0-255)
;
; Notes:
;   Destroys AF,BC,DE,HL,B',DE'
;   Always returns BC = 0
; -----------------------------------------------------------------------------

; calculate icon definition address in HL
        ld      d, 0                    ; DE = icon
        ld      h, d
        ld      l, e                    ; HL = icon
        add     hl, hl                  ; HL = icon*2
        add     hl, hl                  ; HL = icon*4
        add     hl, de                  ; HL = icon*5
        add     hl, hl                  ; HL = icon*10
        add     hl, hl                  ; HL = icon*20
        add     hl, hl                  ; HL = icon*40
        add     hl, de                  ; HL = icon*41
        ld      de, ICONS
        add     hl, de                  ; HL = ICONS + icon*41


; ----------------------------------------------------------------------
; ENTRY POINT #3 - CAN BE DIRECTLY ACCESSED FROM ASSEMBLY ROUTINES
;
; Params:
;   B = row (0-20)
;   C = column (0-28)
;   HL = icon address
;   B' = clip (0-255)
;
; Notes:
;   Destroys AF,BC,DE,HL,B',DE'
;   Always returns BC = 0
; -----------------------------------------------------------------------------

; calculate screen address in DE
        ld      a, b
        or      $47
        ld      d, a                    ; D = %010RR111
        ld      a, b
        and     $07
        rrca
        rrca
        rrca
        or      c
        ld      e, a                    ; E = %rrrccccc

; draw upper quadrants (first 9 pixel lines)
        ld      c, 128                  ; initialize bit group
        call    PIXLINES

; draw lower quadrants (last 9 pixel lines)
        call    PIXLINES

; calculate attribute address
        ld      a, d                    ; A = %010RR001
        rra                             ; A = %0010RR00
        rra                             ; A = %10010RR0
        rra                             ; A = %010010RR
        ld      d, a
        ld      hl, $1000-32-32+1       ; adjust position (2 chars up, 1 char right)
        add     hl, de

; recover clip
        exx
        ld      a, b
        exx
        ld      b, a                    ; B = B'

; update attributes
        ld      de, 1                   ; used to adjust screen address
NEXTATTR:
        sla     c
        ret     z
        ld      a, 56                   ; FLASH 0; BRIGHT 0; PAPER 7; INK 0
        jr      c, OPAQUE
        ld      a, (hl)
        and     $f8                     ; INK 0
OPAQUE:
        srl     b
        jr      c, CLIPPED              ; skip attribute if clipped
        ld      (hl), a
CLIPPED:
        add     hl, de                  ; either next column, or prev column in next row
        ld      a, e
        xor     30                      ; alternate between values 31 and 1
        ld      e, a
        jr      NEXTATTR                ; repeat for next quadrant

; -----------------------------------------------------------------------------
PIXLINES:
; self-modify code depending on pixel clipping bits
        exx
        call    CLIPPING
        ld      (SMC1),a
        ld      (SMC2),de
        call    CLIPPING
        ld      (SMC3),de
        ld      (SMC4),a
        exx

        ld      b, 9                    ; repeat 9 times

NEXTLINE:
; discard previous value at first bit column
        ld      a, (de)
        rra

; recover next bit from icon definition
        sla     c                       ; check next bit
        jr      nz, SKIP
        ld      c, (hl)                 ; load another group of 8 bits
        inc     hl
        rl      c
SKIP:
; put new value at first bit column
        rla
SMC1:
        ld      (de), a                 ; or "NOP" if left side is clipped
        inc     e

; change next 16 bit columns
SMC2:
        inc     bc                      ; or "INC HL" if left quadrant is clipped
        ldi                             ; or "INC DE" / "AND B" if left side is clipped
SMC3:
        inc     bc                      ; or "INC HL" if right quadrant is clipped
        ldi                             ; or "INC DE" / "AND B" if right side is clipped

; discard previous value at last bit column
        ld      a, (de)
        rla

; recover next bit from icon definition
        sla     c                       ; check next bit

; put new value at last bit column
        rra
SMC4:
        ld      (de), a                 ; or "NOP" if right side is clipped

; back to first bit column
        dec     de
        dec     de
        dec     e

; move down 1 pixel line
        inc     d
        ld      a, d
        and     7
        jr      nz, MOVED
        ld      a, e
        add     a, 32
        ld      e, a
        jr      c, MOVED
        ld      a, d
        sub     8
        ld      d, a
MOVED:
        djnz    NEXTLINE                ; repeat for next pixel line
        ret

; -----------------------------------------------------------------------------
CLIPPING:
        ld      a, $12                  ; opcode for "LD (DE),A"
        ld      de, $ed03               ; opcode for "INC BC" / "LDI"
        srl     b
        ret     nc
        xor     a                       ; opcode for "NOP"
        ld      de, $1323               ; opcode for "INC HL" / "INC DE" / "AND B"
        ret

; -----------------------------------------------------------------------------
